package cologne.eck.peafactory.peas.gui;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * View of the pea dialog to type the password. 
 */

import java.awt.*;
import java.awt.event.*;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.net.URL;
import java.util.ResourceBundle;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import settings.PeaSettings;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.peafactory.crypto.KeyDerivation;
import cologne.eck.peafactory.peas.PswDialogBase;
import cologne.eck.peafactory.tools.Attachments;
import cologne.eck.peafactory.tools.EntropyPool;
import cologne.eck.peafactory.tools.KeyRandomCollector;
import cologne.eck.peafactory.tools.MouseRandomCollector;
import cologne.eck.peafactory.tools.Zeroizer;


/*
 * View of PswDialogNotes, PswDialogEditor, PswDialogFile,
 * PswDialogImage 
 */


@SuppressWarnings("serial")
public class PswDialogView extends JDialog implements WindowListener,
		ActionListener, KeyListener {
	
	private static final Color peaColor = new Color(230, 249, 233);
	//private static final Color errorColor = new Color(252, 194, 171);
	
	// screen: 
	private static final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	private static final double screenWidth = screenSize.getWidth();
	private static final double screenHeight = screenSize.getHeight();
	
	// label above password field (if undefined: type password...)
	private static JLabel pswLabel;
	// password field
	private static JPasswordField pswField;
	// Button to start decryption
	private static JButton okButton;
	
	// used for File PEA
	private static JPanel filePanel;
	
	// used for notebook/image: displays all available, valid files to decrypt
	private static JPanel availableFilesPanel;
	// button group for available files
	private static ButtonGroup fileGroup;

	// indicates the type of the PEA (text, file, image...)
	private static String fileType = "dingsda";
	
	// remember the selected files? -> store in path file
	// only for file type
	private static JCheckBox rememberCheck;
	
	// static instance
	private static PswDialogView dialog;
	
	// display warnings and error messages
	private static JLabel errorMessageLabel;
	
	// thread while typing the password - still not used
	private static WhileTypingThread t;
	
	// pea image icon for frames
	private static Image image;
	
	// language support (set in PswDialogBase)
	private static ResourceBundle languagesBundle;
	
	// to check if decryption was startet by ok-button
	private static boolean started = false;
	
	// if started first time (no password, no content)
	// can be set manually in menu
	private static boolean initializing = false; 
	
	// the version number of the current PEA
	private static final String VERSION = " 0.1";
	
	private PswDialogView() {
		
		// start the EntropyPool:
		EntropyPool.getInstance().updateThread(System.nanoTime());
		
		setUI();
		
		languagesBundle = PswDialogBase.getBundle();
		
		fileType = PswDialogBase.getFileType();
	
		dialog = this;
		this.setTitle(PeaSettings.getJarFileName() );
				
		this.setBackground(peaColor);

		URL url = this.getClass().getResource("resources/pea-lock.png");
		if (url == null) {
			try{
				image = new ImageIcon(getClass().getClassLoader().getResource("resources/pea-lock.png")).getImage();
			} catch (Exception e) {
				System.out.println("image not found");
			}
		} else {			
			image = new ImageIcon(url).getImage();
		}


	    this.setIconImage(image);
	
		this.addWindowListener(this);
		
		this.addMouseMotionListener(new MouseRandomCollector() );

		JPanel topPanel = (JPanel) this.getContentPane();
		topPanel.setBackground(peaColor);
		// more border for right and bottom to start
		// EntropyPool if there is no password
		topPanel.setBorder(new EmptyBorder(5,5,15,15));
		
		topPanel.addMouseMotionListener(new MouseRandomCollector() );
		topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
		
		loadComponents(topPanel);
		
		this.setLocation(100,30);
		pack();
	}
	
	public final static PswDialogView getInstance() {
		if (dialog == null) {
			dialog = new PswDialogView();
		} else {
			//return null
		}
		return dialog;
	}
	

	public final void displayErrorMessages(String topic) {

		setMessage("###" + topic + ":\n");
		setMessage(CipherStuff.getErrorMessage());
		PswDialogView.clearPassword();
	}


	@Override
	public void keyPressed(KeyEvent kpe) {
		// EntropyPool must be started by mouse events if initializing
		if(kpe.getKeyChar() == KeyEvent.VK_ENTER && initializing == false){
			okButton.doClick();
		}
	}
	@Override
	public void keyReleased(KeyEvent arg0) {}
	@Override
	public void keyTyped(KeyEvent arg0) {}

	@Override
	public void actionPerformed(ActionEvent ape) {
		String command = ape.getActionCommand();
		//System.out.println("command: " + command);
		if(command.equals("open")){
			//System.out.println("open files");		
			PswDialogBase.getDialog().openEncryptedFiles();
			
		} else if (command.equals("init")){
			//System.out.println("initialize");	
			PswDialogBase.setInsideJar(false);
			initialize();
			
		} else if (command.startsWith("FILE")){			
			
			String fileName = command.substring(4, command.length());
			//System.out.println("selected file: " + fileName);
			// use the file inside the jar archive (resources/text.lock)
			if (fileName.equals(languagesBundle.getString("image_inside_jar_archive"))){
				PswDialogBase.setInsideJar(true);
			} else {
				PswDialogBase.setInsideJar(false);
			}
		    PeaSettings.setExternalFilePath(fileName);
		    PswDialogBase.setEncryptedFileName(fileName);		    
    		
		} else if (command.equals("quit")){
			System.out.println("Exit");
			System.exit(0);
		
		} else if (command.equals("instruction")){
			String message = null;
			if (fileType.equals("file")){
				message = PswDialogBase.getBundle().getString("file_instruction");
			} else if (fileType.equals("image")) {
				message = PswDialogBase.getBundle().getString("image_instruction");
			} else if (fileType.equals("text")) {
				message = PswDialogBase.getBundle().getString("text_instruction");
			} else if (fileType.equals("text file")) {
				message = PswDialogBase.getBundle().getString("text_instruction");
			} else {
				message = "invalid PEA type: " + fileType;
			}
			JOptionPane.showMessageDialog(PswDialogView.getView(),
					message,									
					null, 									   
					JOptionPane.INFORMATION_MESSAGE);

		} else if (command.equals("web")){
			
			showInfoDialog();
			
		} else if(command.equals("keyboard")){
			
			// clear error messages from previous password errors
			if (errorMessageLabel.getText().equals(PswDialogBase.getBundle().getString("password_failed"))){
				errorMessageLabel.setText(" ");
			}
			Keyboard keyboard = new Keyboard(this, pswField);
			keyboard.setVisible(true);
			//PeaSettings.getKeyboard().setVisible(true);
			
		} else if (command.equals("charTable1")){
			CharTable table = new CharTable(this, pswField);
			table.setVisible(true);
			
		} else if (command.equals("ok")){

			boolean rememberPath = false;
			if (fileType.equals("file") && rememberCheck != null) {
				if ( rememberCheck.isSelected() == true){
					rememberPath = true;
				}
			}
			PswDialogBase.getDialog().startExecution(rememberPath);
		}		
	}
	

	@Override
	public void windowActivated(WindowEvent arg0) {}
	@Override
	public void windowClosed(WindowEvent arg0) { // dispose

		this.dispose();

	}
	@Override
	public void windowClosing(WindowEvent arg0) { // x
		
		if (started == true) {
			return;
		} else {
			// if attacker has access to memory but not to file
			if (Attachments.getNonce() != null) {
				Zeroizer.zero(Attachments.getNonce());
			}
			if (PswDialogBase.getDialog() != null) {
				PswDialogBase.getDialog().clearSecretValues();
			}
			System.exit(0);
		}	
	}
	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}

	@Override
	public void windowOpened(WindowEvent arg0) {
		
		// do some expensive computations while user tries to remember the password
		t = this.new WhileTypingThread();
		t.setPriority( Thread.MAX_PRIORITY );// 10
		t.start();
		if (initializing == false) {
			pswField.requestFocus();
		}
	}

	
	//=======================================
	// Helper Functions
	
	public final static void setUI() {
//		UIManager.put("Button.background", peaColor );					
//		UIManager.put("MenuBar.background", peaColor ); // menubar
//		UIManager.put("MenuItem.background", color2 ); // menuitem
		UIManager.put("PopupMenu.background", peaColor );	//submenu									
		UIManager.put("OptionPane.background", peaColor );
		UIManager.put("Panel.background", peaColor );
		UIManager.put("RadioButton.background", peaColor );
		UIManager.put("ToolTip.background", peaColor );
		UIManager.put("CheckBox.background", peaColor );	
		
		UIManager.put("InternalFrame.background", peaColor );
		UIManager.put("ScrollPane.background", peaColor );
		UIManager.put("Viewport.background", peaColor );	
		// ScrollBar.background  Viewport.background
		
		// don't use BOLD for messages, some messages contain lots of text
		UIManager.put("OptionPane.messageFont", new Font("System", Font.PLAIN, 14));
	}
	
	private static void loadComponents(JPanel topPanel) {
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		
		JMenuBar menuBar = new JMenuBar();
		menuBar.setBorder(new EtchedBorder() );	
		
		JMenu menu = new JMenu(languagesBundle.getString("file"));
		menu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
//		menu.setMnemonic(KeyEvent.VK_M);
		menu.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("file_mnemonic")).getKeyCode());
		menuBar.add(menu);
		
		JMenuItem openItem = new JMenuItem(languagesBundle.getString("open_encrypted_file"));
		openItem.setActionCommand("open");
		openItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//openItem.setMnemonic(KeyEvent.VK_O);
		openItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("open_mnemonic")).getKeyCode());
		openItem.addActionListener(dialog);
		menu.add(openItem);
		
		JMenuItem initItem = new JMenuItem(languagesBundle.getString("init_new_file"));
		initItem.setActionCommand("init");
		initItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//initItem.setMnemonic(KeyEvent.VK_I);
		initItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("init_mnemonic")).getKeyCode());
		initItem.addActionListener(dialog);
		menu.add(initItem);		
				

		JMenuItem quitItem = new JMenuItem(languagesBundle.getString("quit"));//"Quit");
		quitItem.setActionCommand("quit");
		quitItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//quitItem.setMnemonic(KeyEvent.VK_Q);
		quitItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("quit_mnemonic")).getKeyCode());
		quitItem.addActionListener(dialog);
		menu.add(quitItem);		
		
		
		JMenu extrasMenu = new JMenu(languagesBundle.getString("extras"));
		extrasMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//extrasMenu.setMnemonic(KeyEvent.VK_M);
		extrasMenu.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("extras_mnemonic")).getKeyCode());
		menuBar.add(extrasMenu);
				
		JMenuItem charTableItem = new JMenuItem(languagesBundle.getString("char_table"));
		charTableItem.setActionCommand("charTable1");
		charTableItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//charTableItem.setMnemonic(KeyEvent.VK_I);
		charTableItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("char_table_mnemonic")).getKeyCode());
		charTableItem.addActionListener(dialog);
		extrasMenu.add(charTableItem);
	
		JMenuItem keyboardItem = new JMenuItem(languagesBundle.getString("keyboard"));
		keyboardItem.setActionCommand("keyboard");
		keyboardItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//keyboardItem.setMnemonic(KeyEvent.VK_I);
		keyboardItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("keyboard_mnemonic")).getKeyCode());
		keyboardItem.addActionListener(dialog);
		extrasMenu.add(keyboardItem);
		
		
		JMenu help = new JMenu(languagesBundle.getString("help"));
		help.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//help.setMnemonic(KeyEvent.VK_H);
		help.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("help_mnemonic")).getKeyCode());
		menuBar.add(help);
		
		JMenuItem instructionItem = new JMenuItem(languagesBundle.getString("instruction"));
		instructionItem.setActionCommand("instruction");
		instructionItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//instructionItem.setMnemonic(KeyEvent.VK_I);
		instructionItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
		instructionItem.addActionListener(dialog);
		help.add(instructionItem);
		
		JMenuItem webItem = new JMenuItem(languagesBundle.getString("web"));
		webItem.setActionCommand("web");
		webItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//webItem.setMnemonic(KeyEvent.VK_I);
		webItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("web_mnemonic")).getKeyCode());
		webItem.addActionListener(dialog);
		help.add(webItem);
		
		menuPanel.add(menuBar);
		menuPanel.add(Box.createHorizontalGlue());
		topPanel.add(menuPanel);
		
		topPanel.add(Box.createVerticalStrut(5));
		
		JPanel errorMessagePanel = new JPanel();
		errorMessagePanel.setLayout(new BoxLayout(errorMessagePanel, BoxLayout.X_AXIS));		
		errorMessageLabel = new JLabel("   ");
		errorMessageLabel.setForeground(Color.RED);
		errorMessagePanel.add(errorMessageLabel);
		errorMessagePanel.add(Box.createHorizontalGlue());
		topPanel.add(errorMessagePanel);
		topPanel.add(Box.createVerticalStrut(5));
		
		okButton = new JButton(languagesBundle.getString("ok"));
		okButton.setPreferredSize(new Dimension( 60, 30));
		okButton.setActionCommand("ok");
		okButton.addActionListener(dialog);
		// if there is no password, this might be the only 
		// chance to start EntropyPool
		okButton.addMouseMotionListener(new MouseRandomCollector() );
		
		if (initializing == false) {		
			
			if (PswDialogBase.getWorkingMode().equals("-t")) { // test mode
				JLabel rescueLabel = new JLabel("TEST MODE");
				topPanel.add(rescueLabel);
				try { //redirect stderr to file
					System.setErr(new PrintStream(new FileOutputStream("error_log.txt")));
					// print some informations to stderr:
					System.err.println(System.getProperty("os.name") + "\n"
							+ "Java " + System.getProperty("java.version") + "\n"
							+ "PEA type: " + PswDialogBase.getFileType() + ", version: " + VERSION);
					System.err.println(KeyDerivation.getKdf().getName() + ", " 
							+ CipherStuff.getCipherAlgo().getAlgorithmName() 
							+ ", " + HashStuff.getHashAlgo().getAlgorithmName());
				} catch (Exception e) {
					System.err.println("Redirection of stderr failed");
				}
				
				if (PswDialogBase.getFileType().equals("image")) { // test mode

					String names[] = ImageIO.getReaderFormatNames();
					StringBuilder builder = new StringBuilder();
					for (int i = 0; i < names.length; ++i) {
						builder.append("\"" + names[i] + "\",");
					//System.out.println(names[i]);
					}
					//System.out.println(builder.substring(0, builder.lastIndexOf(",")));
					System.err.println("Supported image formats: " +  builder.substring(0, builder.lastIndexOf(",")));// delete the last ','
					
				}
			} else if (PswDialogBase.getWorkingMode().equals("-r")) { // rescue mode
				JLabel rescueLabel = new JLabel(languagesBundle.getString("rescue_mode"));
				rescueLabel.setForeground(Color.RED);
				topPanel.add(rescueLabel);
			}

			if (fileType.equals("file") ) {
				filePanel = PswDialogBase.getTypePanel();
				filePanel.addMouseMotionListener(new MouseRandomCollector() );
				topPanel.add(Box.createVerticalStrut(10));
				topPanel.add(filePanel);
				
			} else { // text, passwordSafe, image: one single file

				if(availableFilesPanel == null){

					availableFilesPanel = new JPanel();
					availableFilesPanel.setLayout(new BoxLayout(availableFilesPanel, BoxLayout.Y_AXIS));
					fileGroup = new ButtonGroup();
				}
				topPanel.add(availableFilesPanel);
				
				PswDialogBase.searchSingleFile();					
			}

			topPanel.add(Box.createVerticalStrut(5));
			
			JPanel pswLabelPanel = new JPanel();
			pswLabelPanel.addMouseMotionListener(new MouseRandomCollector() );
			pswLabelPanel.setLayout(new BoxLayout(pswLabelPanel, BoxLayout.X_AXIS));
			topPanel.add(Box.createVerticalStrut(10));
			topPanel.add(pswLabelPanel);
			
			if (PeaSettings.getLabelText() == null) {
				pswLabel = new JLabel( languagesBundle.getString("enter_password") );
			} else {
				pswLabel = new JLabel( PeaSettings.getLabelText() );
			}
			pswLabel.setPreferredSize(new Dimension( 460, 20));
			pswLabelPanel.add(pswLabel);
			pswLabelPanel.add(Box.createHorizontalGlue());
			
			JPanel pswPanel = new JPanel();
			pswPanel.setLayout(new BoxLayout(pswPanel, BoxLayout.X_AXIS));
			
			pswField = new JPasswordField();
			pswField.setBackground(new Color(231, 231, 231) );
			pswField.setPreferredSize(new Dimension(400, 30));
			pswField.addKeyListener(dialog);
			pswField.addKeyListener(new KeyRandomCollector() );

			pswPanel.add(pswField);
			
			pswPanel.add(okButton);
			topPanel.add(pswPanel);
		
		} else {// initializing
			//
		}		
		
		if (fileType.equals("file")){		
			
			JPanel checkRememberPanel = new JPanel();
			checkRememberPanel.setLayout(new BoxLayout(checkRememberPanel, BoxLayout.X_AXIS));
			
		    rememberCheck = new JCheckBox();	
		    rememberCheck.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		    rememberCheck.setText(languagesBundle.getString("remember_file_names"));//"remember selected file names");
		    rememberCheck.setToolTipText(languagesBundle.getString("file_storage_info")// "file names will be saved in the file " +
		    		+  "  "  + PeaSettings.getJarFileName() + ".path");// in current directory");
			
		    checkRememberPanel.add(Box.createVerticalStrut(10));
		    checkRememberPanel.add(rememberCheck);
		    checkRememberPanel.add(Box.createHorizontalGlue() );
		    
		    topPanel.add(checkRememberPanel);
		}		
	}

	
	public final static void updateView(){
		
		dialog.getContentPane().removeAll();
		loadComponents((JPanel) dialog.getContentPane());
		dialog.pack();
	    
	/*    availableFilesPanel.revalidate();
	    dialog.getContentPane().revalidate();
	    dialog.revalidate();
		dialog.pack();	
		dialog.repaint();	  */
	}
	
	/**
	 * Initialize the PEA with a new password, new salt and new content
	 */
	protected final static void initialize(){
		
		initializing = true;
		
		NewPasswordDialog.setRandomCollector(true);
		String hint = null;
		if (PswDialogBase.getFileType().equals("file")){
			hint = PswDialogBase.getBundle().getString("hint_initialization");
		}
		NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(
				dialog, PswDialogBase.getBundle().getString("initialization"),
		hint);

		PswDialogBase.getDialog().setInitializedPassword( newPswDialog.getDialogInput() );

		NewPasswordDialog.setRandomCollector(false);

		if (newPswDialog.getDialogInput() == null) {
			System.exit(0);
		}		
		PswDialogBase.getDialog().initializeNewFiles();
		
		dialog.setVisible(false);

		PswDialogBase.getDialog().startExecution(false);
	}	
	
	//===============================================
	// Getter & Setter
	public final static char[] getPassword() {
		return pswField.getPassword();
	}
	protected final static void setPassword(char[] psw) { // for Keyboard
		pswField.setText( new String(psw) );
	}
	protected final static void setPassword(char pswChar) { // for charTable
		pswField.setText("" +  pswChar );
	}
	public final static void clearPassword() {
		if (pswField != null) {
			pswField.setText("");
		}
	}
	public final static void setMessage(String message) {
		//System.out.println("PswDialogView - setMessage: " + message);
		if (message != null) {
		
			if (message.startsWith("###")){
				errorMessageLabel.setText(message.substring(3, message.length()));
			} else {
				try {
					String i18nMessage = languagesBundle.getString(message);
					errorMessageLabel.setText(i18nMessage);
				} catch (Exception e) {
					errorMessageLabel.setText(message);
				}
			}	
		}
	}
	public final static void setLabelText(String labelText) {
		if (initializing == true){
			pswLabel.setText(labelText);
		}
	}
	public final static boolean validFileAvailable() {
		if (fileGroup == null){
			return false;
		}
		if (fileGroup.getButtonCount() > 0){
			return true;
		} else {
			return false;
		}
	}
	public final static void addAvailableFile(String fileName, boolean newSelected) {
		
		JPanel fileRadioPanel = new JPanel();
		fileRadioPanel.setLayout(new BoxLayout(fileRadioPanel, BoxLayout.X_AXIS));
		JRadioButton fileRadioButton = new JRadioButton();

		// if first file: select
		if (fileGroup.getButtonCount() == 0){
			fileRadioButton.setSelected(true);			
			
			if (fileName.equals(languagesBundle.getString("image_inside_jar_archive"))){
				PswDialogBase.setInsideJar(true);
			} else {
				PswDialogBase.setInsideJar(false);
				PswDialogBase.setEncryptedFileName(fileName);
			}
		}

		fileGroup.add(fileRadioButton);
		
		fileRadioButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		fileRadioButton.setText(fileName);//"remember selected file names");
		fileRadioButton.addActionListener(dialog);
		fileRadioButton.setActionCommand("FILE" + fileName);
		
	    availableFilesPanel.add(Box.createVerticalStrut(3));
	    fileRadioPanel.add(fileRadioButton);
	    fileRadioPanel.add(Box.createHorizontalGlue() );

	    availableFilesPanel.add(fileRadioPanel);
	    
	    if (newSelected == true) {

	    	fileRadioButton.setSelected(true);
	    	// set selected file
			if (fileName.equals(languagesBundle.getString("image_inside_jar_archive"))){
				PswDialogBase.setInsideJar(true);
			} else {
				PswDialogBase.setInsideJar(false);
			}
		    PswDialogBase.setEncryptedFileName(fileName);	    	
	    			    
		    availableFilesPanel.revalidate();
		    dialog.revalidate();
		    dialog.repaint();
		    dialog.pack();
		    
    		errorMessageLabel.setText(" ");
	    }
	}
	
	public static void customizeFileChooser(String titleText, String instructionText, 
			String locationText, String fileNameText, JFileChooser chooser) {

		if (titleText != null) {
			chooser.setDialogTitle(titleText);
		}
		
		JPanel centerPanel = null;
		
		if (instructionText != null) {
			// add message to the file chooser:
			JPanel northPanel = new JPanel();
			northPanel.setLayout(new BoxLayout(northPanel, BoxLayout.Y_AXIS));
			JPanel messagePanel = new JPanel();
			
			messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.X_AXIS));
			JLabel messageLabel = new JLabel(instructionText);
			//messageLabel.setForeground(new Color(153, 15, 15));//Color.RED);
			messageLabel.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, 18));
			messagePanel.add(messageLabel);
			messagePanel.add(Box.createHorizontalGlue());
			
			northPanel.add(messagePanel);
			northPanel.add(Box.createVerticalStrut(20));
			// add existing component at PAGE_START
			northPanel.add( chooser.getComponent(0));
			chooser.add(northPanel, BorderLayout.PAGE_START);
		}
		if (locationText != null) {
			// add message to the file chooser:
			centerPanel = new JPanel();
			centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
			JPanel messagePanel = new JPanel();
			messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.X_AXIS));
			JLabel messageLabel = new JLabel(locationText);
			messageLabel.setForeground(new Color(153, 15, 15));
			messageLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
			messagePanel.add(messageLabel);
			messagePanel.add(Box.createHorizontalGlue());	
			centerPanel.add(Box.createVerticalStrut(10));
			centerPanel.add(messagePanel);
			centerPanel.add(Box.createVerticalStrut(5));
			// add existing component 
			centerPanel.add( chooser.getComponent(1));
		}
		if (fileNameText != null) {
			// add message to the file chooser:
			JPanel southPanel = new JPanel();
			southPanel.setLayout(new BoxLayout(southPanel, BoxLayout.Y_AXIS));
			JPanel messagePanel = new JPanel();
			messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.X_AXIS));
			JLabel messageLabel = new JLabel(fileNameText);
			messageLabel.setForeground(new Color(153, 15, 15));
			messageLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 18));
			messagePanel.add(messageLabel);
			messagePanel.add(Box.createHorizontalGlue());
			southPanel.add(messagePanel);
			
			if (centerPanel == null) {
				centerPanel = new JPanel();
				centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
				// add existing component 
				centerPanel.add( chooser.getComponent(1));
			}
			
			// add as last panel to centerPanel
			centerPanel.add(Box.createVerticalStrut(20));
			centerPanel.add(southPanel);

		}
		if (centerPanel != null) {
			chooser.add(centerPanel, BorderLayout.CENTER);
		}
	}
	/**
	 * Show a message dialog with links and email, related to this PEA
	 * 
	 * @param websiteFolder		folder with leading slash after the language,
	 * 							that is "/html/xxx_pea.html"
	 * @param downloadLink		full link to download the source code
	 */
	public final static void showInfoDialog(){
		
		String websiteFolder = PswDialogBase.getDialog().getWebsite();
		String downloadLink = PswDialogBase.getDialog().getSourceLink();
		
		String lang = System.getProperty("user.language"); // en, de
		if (lang != "de" && lang != "en") {
			lang = "en";
		}
		String infoSite = "http://eck.cologne/peafactory/" + lang + websiteFolder;
		

		JOptionPane.showMessageDialog(PswDialogView.getView(),
				PswDialogBase.getBundle().getString("find_web_infos") + ": \n" 
						+ "     " + infoSite + "\n\n" 
						+ PswDialogBase.getBundle().getString("general_infos") + ": \n" 
						+ "   http://eck.cologne/peafactory/" + lang + "/index.html\n\n" 
						+ PswDialogBase.getBundle().getString("find_source_code") + ": \n" 
						+ "     " + downloadLink + "\n\n" 
						+ PswDialogBase.getBundle().getString("contact") + ": \n" 
						+ "   peafactory@eck.cologne\n",									
						PswDialogBase.getBundle().getString("help"), 									   
						JOptionPane.INFORMATION_MESSAGE);	
	}
	/**
	 * Show a dialog with informations about the properties of the current PEA
	 * 
	 * @param peaType		the type or name of the PEA
	 * @param extraLine		an optional extra line for PEA dependent properties
	 */
	public final static void showAboutDialog(String peaType, String extraLine) {
		
		String x = PswDialogBase.getBundle().getString("with") + " ";
		if (PeaSettings.getKeyFileProperty() == false){
			x = PswDialogBase.getBundle().getString("without") + " ";
		}
		if (extraLine == null){
			extraLine = " ";
		}
		JOptionPane.showMessageDialog(
				PswDialogView.getView(),
				peaType + " - " + PswDialogBase.getBundle().getString("part_of") + " PeaFactory\n" 
				+ PswDialogBase.getBundle().getString("version") + ": " + PswDialogBase.getDialog().getVersion() + " (" + PswDialogBase.getDialog().getYearOfPublication() + ")\n" +
				x + PswDialogBase.getBundle().getString("key_file_property") + "\n" + 
				extraLine + "\n"
				+ PswDialogBase.getBundle().getString("used_primitives") + "\n"
				+ "Key derivation: " +  KeyDerivation.getKdf().getName() 
				+ ", time parameter: " + KeyDerivation.gettCost() 
				+ ", memory parameter: " + KeyDerivation.getmCost() + "\n"
				+ "Cipher: " + CipherStuff.getCipherAlgo().getAlgorithmName() + "\n"
				+ "Mode: " + CipherStuff.getCipherMode().getClass().getSimpleName() + "\n"
				+ "Hash Function: " + HashStuff.getHashAlgo().getAlgorithmName(),
				null, JOptionPane.PLAIN_MESSAGE);
	}
	
	
	public final static WhileTypingThread getThread() {
		return t;
	}
	public final static Image getImage() {
	    return image;
	}
	public final static PswDialogView getView() {
		return dialog;
	}
	public final JPanel getFilePanel() {
		return filePanel;
	}
	public final static Color getPeaColor(){
		return peaColor;
	}
	public final static void setStarted(boolean _started) {
		started = _started;
	}
	
	/**
	 * @return the initializing
	 */
	public static boolean isInitializing() {
		return initializing;
	}

	/**
	 * @param initializing the initializing to set
	 */
	public static void setInitializing(boolean initialize) {
		PswDialogView.initializing = initialize;
	}

	/**
	 * @return the screenwidth
	 */
	public static double getScreenwidth() {
		return screenWidth;
	}

	/**
	 * @return the screenheight
	 */
	public static double getScreenheight() {
		return screenHeight;
	}

	//================================================
	// inner class
	public class WhileTypingThread extends Thread {
		// some pre-computations while typing the password
		@Override
		public void run() {		
			if (PswDialogBase.getDialog() != null) {
				PswDialogBase.getDialog().preComputeInThread();
			}
		}
	}
}