package cologne.eck.peafactory.peas.image_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main view of image pea. 
 */

/*
 * Frame to display images, decrypted in PswDialogImage
 * (this is not a daughter class of LockFrame!)
 * 
 */

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.EtchedBorder;

import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.peas.PswDialogBase;
import cologne.eck.peafactory.peas.gui.LockFrame;
import cologne.eck.peafactory.peas.gui.PswDialogView;
import cologne.eck.peafactory.tools.ReadResources;
import cologne.eck.peafactory.tools.WriteResources;
import cologne.eck.peafactory.tools.Zeroizer;

@SuppressWarnings("serial")
final class LockFrameImage extends LockFrame implements ActionListener, ImageObserver {

	// the loaded image as an array of bytes
	private static byte[] imageBytes;
	
	// check if there is already an instance
	private static boolean isInstantiated = false;
	
	// the image to show
	private static BufferedImage image = null;		
	// scroll pane containing the image
	private JScrollPane scrollPane = null;
	// the label containing the scroll pane
	private JLabel picLabel = null;

	// zoom value for width: additional width
	private static int zoomWidth = 0;
	// zoom value for height: additional height
	private static int zoomHeight = 0;
	
	// resized default width from PswDialogImage
	private int imgWidth = 0;
	// resized default height from PswDialogImage
	private int imgHeight = 0;

	
	private LockFrameImage() {
		
	    this.setIconImage(PswDialogView.getImage());
		
		this.addWindowListener(this);// for windowClosing
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		this.setContentPane(contentPane);		
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		menuPanel.setMaximumSize(new Dimension(600, 30));
		
		JMenuBar menuBar = new JMenuBar();
		menuBar.setBorder(new EtchedBorder() );
		
		JMenu menu = new JMenu();
		menu.setText(PswDialogBase.getBundle().getString("menu"));//"Menu");
		menu.setMnemonic(KeyEvent.VK_M);
		menuBar.add(menu);
		
		JMenuItem encryptItem = new JMenuItem();
		encryptItem.setText(PswDialogBase.getBundle().getString("close_encrypted"));
		encryptItem.setActionCommand("encrypted");
		encryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		encryptItem.setMnemonic(KeyEvent.VK_E);
		encryptItem.addActionListener(this);
		menu.add(encryptItem);

		
		// if image is inside the jar, there is no possibility to chnage the
		// password or to leave the image unencrypted
		if (PswDialogBase.isInsideJar() == false) {
			JMenuItem unencryptItem = new JMenuItem();
			unencryptItem.setText(PswDialogBase.getBundle().getString("close_unencrypted"));
			unencryptItem.setActionCommand("unencrypted");
			unencryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
			unencryptItem.setMnemonic(KeyEvent.VK_U);
			unencryptItem.addActionListener(this);
			menu.add(unencryptItem);
			
			JMenuItem pswChangeItem = new JMenuItem();
			pswChangeItem.setText(PswDialogBase.getBundle().getString("change_password"));//"change password");
			pswChangeItem.setActionCommand("changePassword");
			pswChangeItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
			pswChangeItem.setMnemonic(KeyEvent.VK_P);
			pswChangeItem.addActionListener(this);
			menu.add(pswChangeItem);
		}
		
		JMenu helpMenu = new JMenu();
		helpMenu.setText(PswDialogBase.getBundle().getString("help"));
		helpMenu.setMnemonic(KeyEvent.VK_H);
		menuBar.add(helpMenu);
		
		JMenuItem aboutItem = new JMenuItem(PswDialogBase.getBundle().getString("about"));
		aboutItem.setActionCommand("about");
		aboutItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		aboutItem.setMnemonic(KeyEvent.VK_A);
		aboutItem.addActionListener(this);
		helpMenu.add(aboutItem);

		JMenuItem homepageItem = new JMenuItem(PswDialogBase.getBundle().getString("infos"));
		homepageItem.setActionCommand("infos");
		homepageItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		homepageItem.setMnemonic(KeyEvent.VK_H);
		homepageItem.addActionListener(this);
		helpMenu.add(homepageItem);

		menuPanel.add(menuBar);
		menuPanel.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		
		// get image from file name:		
		try {			
/*			// compressed image with less quality
			BufferedImage fullImage = ImageIO.read(new ByteArrayInputStream(imageBytes));
            image = new BufferedImage(imgWidth, imgHeight,
                    BufferedImage.TYPE_INT_ARGB);
                Graphics2D g = image.createGraphics();
                g.drawImage(fullImage, 0, 0, imgWidth, imgHeight, this);
                g.dispose(); */			
			image = ImageIO.read(new ByteArrayInputStream(imageBytes));

		} catch (IOException e) {			
			System.err.println("LockFrameImage: " + e.toString() + " - " + e.getMessage());
			if (PswDialogBase.getWorkingMode().equals("-t")) { // test mode				
				e.printStackTrace();
			} 
		}
		
		if (image ==  null) { // invalid file
			JOptionPane.showMessageDialog(this, 
					PswDialogBase.getBundle().getString("invalid_image_file")//"Invalid or unsuitable image file: \n"                      
					      + PswDialogImage.getEncryptedFileName(), 
					      PswDialogBase.getBundle().getString("error"),                                
					   JOptionPane.ERROR_MESSAGE);   
			System.exit(1);
		}
		// resize the image
		int[] newSize = resizeImage(image, PswDialogImage.getImageWidth(), PswDialogImage.getImageHeight());
		imgWidth = newSize[0];
		imgHeight = newSize[1];	
		
		// check if image is larger than default values for width and height: 
		if (image.getWidth() > imgWidth || image.getHeight() > imgHeight){
			// compress the image:
			BufferedImage  resizedImage = new BufferedImage(imgWidth + zoomWidth, imgHeight + zoomHeight,
	        BufferedImage.TYPE_INT_ARGB);
	        Graphics2D g = resizedImage.createGraphics();
	        g.drawImage(image, 0, 0, imgWidth + zoomWidth, imgHeight + zoomHeight, this);              
	        g.dispose();	
	        
			picLabel = new JLabel(new ImageIcon(resizedImage));
			scrollPane = new JScrollPane(picLabel);
			scrollPane.setPreferredSize(new Dimension(imgWidth, imgHeight));
		} else {
			// do not compress
			picLabel = new JLabel(new ImageIcon(image));
			scrollPane = new JScrollPane(picLabel);
			scrollPane.setPreferredSize(new Dimension(imgWidth, imgHeight));
			imgWidth = image.getWidth();
			imgHeight = image.getHeight();
		}
		
		// set maximum size, depending on screen size 
		scrollPane.setMaximumSize(new Dimension (
						(int)PswDialogView.getScreenwidth() - 100, 
						(int) PswDialogView.getScreenheight() - 200 ));// think of buttons
		contentPane.add(scrollPane);
		
		JPanel zoomPanel = new JPanel();
		zoomPanel.setMaximumSize(new Dimension(400, 30));
		zoomPanel.setMinimumSize(new Dimension(400, 30));
		
		JButton zoomInButton = new JButton ("+");
		zoomInButton.setActionCommand("zoomIn");
		zoomInButton.addActionListener(this);
		zoomPanel.add(zoomInButton);	
		
		JButton zoomOutButton = new JButton ("-");
		zoomOutButton.setActionCommand("zoomOut");
		zoomOutButton.addActionListener(this);
		zoomPanel.add(zoomOutButton);	
		
		zoomPanel.add(zoomInButton);
		zoomPanel.add(zoomOutButton);
		
		contentPane.add(zoomPanel);		

		this.setLocation(100, 50);
		
		// pack or setMaximumSize/MinimumSize depending on screen size and image size
		if (this.getPreferredSize().getWidth() > PswDialogView.getScreenwidth() - 100
			|| this.getPreferredSize().getHeight() > PswDialogView.getScreenheight() - 100) {
			
			this.setMaximumSize(
					new Dimension (
							(int)PswDialogView.getScreenwidth() - 100,
							(int)PswDialogView.getScreenheight() - 100));
			int w = PswDialogImage.getImageWidth();
			if (w > PswDialogView.getScreenwidth() - 100){
				w = (int) (PswDialogView.getScreenwidth() - 100);
			}
			int h = PswDialogImage.getImageHeight();
			if (h > PswDialogView.getScreenheight() - 100){
				h = (int) (PswDialogView.getScreenheight() - 100);
			}
			// is is the shown size if the screen is large enough
			this.setMinimumSize(new Dimension (w, h));			
		} else {
			
			pack();
		}
		isInstantiated = true;
	}
	/**
	 * Returns an already instantiated instance 
	 * 
	 * @return 
	 */
	protected final static LockFrameImage getInstance() {
		LockFrameImage frame = null;
		if (isInstantiated == false) {
			frame = new LockFrameImage();
		} else {
			//return null
		}		
		return frame;
	}
	 // just to test the view	 
	public static void main(String[] args) {

		PswDialogView.setUI();
		//LockFrameImage.setImageBytes(ReadResources.readExternFile("resources" + File.separator + "Phahlbaudorf.JPG"));
		//LockFrameImage.setImageBytes(ReadResources.readExternFile("resources" + File.separator + "olympos.JPG"));
		//LockFrameImage.setImageBytes(ReadResources.readExternFile("resources" + File.separator + "quer.gif"));
		//LockFrameImage.setImageBytes(ReadResources.readExternFile("resources" + File.separator + "vertikal.bmp"));
		LockFrameImage.setImageBytes(ReadResources.readExternFile("resources" + File.separator + "pea-lock.png"));
		LockFrameImage frame = new LockFrameImage();
		
		frame.setVisible(true);
	} 
	
	/**
	 * The method resizes an image to keep the ratio of width and height
	 * to avoid distortions of the image
	 * 
	 * @param image		the image to resize
	 * @param width		the width of the frame to display the image
	 * @param height	the height of the frame to display the image
	 * 
	 * @return			resulting width ad height, stored in an array:
	 * 					result[0] is the new width, result[1] the new height to display
	 */
	private final int[] resizeImage(BufferedImage image, int width, int height){
		// keep ratio of image width and height: resize the image 
		int w = width;
		int h = height;			
		double op = 0;		 
		 
		 if (image.getWidth() < width) {
			 op = (double)width / image.getWidth();
			 if (image.getHeight() * op > height) op = (double)height / image.getHeight();
			 w = (int) (image.getWidth() * op);
			 h = (int) (image.getHeight() * op);
		 } else {
			 op = image.getWidth() / (double)width;
			 if ( (image.getHeight() / op) > height) op = image.getHeight() / (double)height;
			 w = (int) (image.getWidth() / op);
			 h = (int) (image.getHeight() / op);
		 }	
		 int[] result = new int[2];
		 result[0] = w;
		 result[1] = h;
		 return result;
	}

	/**
	 * Zoom in or out: Resize the image, display the image and set the scroll bars 
	 * 
	 * @param in	if true: zoom in, otherwise zoom out
	 */
	private final void zoom(boolean in){
		
		  int previouseV = scrollPane.getVerticalScrollBar().getValue();
		  int previouseH = scrollPane.getHorizontalScrollBar().getValue();
		
		if (in == true) {			

			zoomWidth += imgWidth/2;
			zoomHeight += imgHeight/2;

/*			// this does not work reliably to prevent OutOfMemoryError;
			// because gc will run, the free memory differs and zoom is
			// stopped before limit is reached
			int typeMultiplicator = 8; // color type: TYPE_INT_ARGB	
			long usedHeap = (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) ;			
			System.out.println("used memory : " + usedHeap);
			System.out.println("free memory: " + Runtime.getRuntime().freeMemory() );
			System.out.println("multiplicator: " + typeMultiplicator + ", image size: " + imgWidth + " * " + imgHeight + ", zoom w*h: " + + zoomWidth + " * " + zoomHeight);
			System.out.println("needed space for image: " + ((imgWidth + zoomWidth) * (imgHeight + zoomHeight) * typeMultiplicator));
			// avoid java.lang.OutOfMemoryError: if image is larger than free heap
			if (((imgWidth + zoomWidth) * (imgHeight + zoomHeight) * typeMultiplicator) >= (Runtime.getRuntime().totalMemory() - usedHeap)) {
				zoomWidth -= imgWidth/2;
				zoomHeight -= imgHeight/2;	
				return;
			}*/
			
		} else {
			
			zoomWidth -= imgWidth/2;
			zoomHeight -= imgHeight/2;	
			
			if (imgWidth + zoomWidth <= 0 || imgHeight + zoomHeight <= 0){
				//System.out.println("Zoom out: invalid size values for image.");				
				zoomWidth += imgWidth/2;
				zoomHeight += imgHeight/2;
				// avoid IllegalArgumentException: do not paint the image
				return;
			}
		}
		try {
			// new compressed/resized image
			BufferedImage  compressedImage = new BufferedImage(
					imgWidth + zoomWidth, imgHeight + zoomHeight,
                  BufferedImage.TYPE_INT_ARGB);
            Graphics2D g = compressedImage.createGraphics();
            g.drawImage(image, 0, 0, imgWidth + zoomWidth, imgHeight + zoomHeight, this);              
            g.dispose();		
		  
             
            picLabel.setIcon(new ImageIcon(compressedImage));
               
      		if (in == true) {      	
                scrollPane.getVerticalScrollBar().setValue((int) (previouseV + imgHeight/4));
                scrollPane.getHorizontalScrollBar().setValue((int) (previouseH + imgWidth/4));                
      		} else {
                scrollPane.getVerticalScrollBar().setValue((int) (previouseV - imgHeight/4));
                scrollPane.getHorizontalScrollBar().setValue((int) (previouseH - imgWidth/4));
      		}
		} catch (IllegalArgumentException e){  // width and height are < 0

			if (PswDialogBase.getWorkingMode().equals("-t")) { // test mode
				System.out.println(e.toString() + ", " + e.getMessage());
			}		 
			// reset the zoom
			if (in == true) {
				 zoomWidth -= PswDialogImage.getImageWidth() / 2;
				 zoomHeight -= PswDialogImage.getImageHeight() / 2;
			} else {
				 zoomWidth += PswDialogImage.getImageWidth() / 2;
				 zoomHeight += PswDialogImage.getImageHeight() / 2;		
			}
		} catch (OutOfMemoryError ex) { // ... if the image is too large
			if (PswDialogBase.getWorkingMode().equals("-t")) { // test mode
				System.err.println(ex.toString() + " - " + ex.getMessage());
			  
				System.err.println("total memory: " + Runtime.getRuntime().totalMemory());
				System.err.println("free memory: " + Runtime.getRuntime().freeMemory());
				// used space: image width * image height * 8 (TYPE_INT_ARGB) 
				System.err.println("used space for image: " + ((imgWidth + zoomWidth) * (imgHeight + zoomHeight) * 8));
			}		  
			zoomWidth -= imgWidth/2;
			zoomHeight -= imgHeight/2;
		}
	}

	/**
	 * Set the image to show. This class holds a copy of the image as an array of bytes
	 * 
	 * @param input		the image as an array of bytes
	 */
	protected final static void setImageBytes( byte[] input ) {
		imageBytes = new byte[input.length];
		System.arraycopy(input,  0,  imageBytes,  0,  input.length);
	}

	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();

		if (command.equals("unencrypted")) {
			WriteResources.write(imageBytes, PswDialogBase.getEncryptedFileName(), null );
			Zeroizer.zero(imageBytes);
			System.exit(0);
			
		} else if (command.equals("encrypted")){
			// file was not modified, just loaded unencrypted in RAM
			Zeroizer.zero(imageBytes);
			System.exit(0);
		} else if (command.equals("changePassword")){
			String[] fileNames = {PswDialogBase.getEncryptedFileName()};
			changePassword(imageBytes, fileNames);
		} else if (command.equals("zoomIn")){
			zoom(true);
		} else if (command.equals("zoomOut")){
			zoom(false);
		} else if (command.equals("about")) {
			PswDialogView.showAboutDialog("Image Lock PEA", PswDialogBase.getBundle().getString("supported_formats") + ": png, jpg, gif, wbmp");
			
		} else if (command.equals("infos")){			
			
			PswDialogView.showInfoDialog();
		}
	}


	@Override
	protected void windowClosingCommands() {
		CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
		Zeroizer.zero(imageBytes);
		System.exit(0);
	}

	@Override
	protected byte[] getPlainBytes() {
		return null;
	}
}