package cologne.eck.peafactory.peas.image_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Dialog to get the password and start the image pea. 
 */

/*
 * Password dialog to decrypt image. 
 * Image is displayed in LockFrameImage. 
 * 
 * Password can not be changed and content (image) can not be modified. 
 * 
 * cipher text is stored in external file or inside jar
 * 
 */


import java.io.File;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import settings.PeaSettings;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.KeyDerivation;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.peafactory.peas.PswDialogBase;
import cologne.eck.peafactory.peas.gui.LockFrame;
import cologne.eck.peafactory.peas.gui.NewPasswordDialog;
import cologne.eck.peafactory.peas.gui.PswDialogView;
import cologne.eck.peafactory.tools.Attachments;
import cologne.eck.peafactory.tools.EntropyPool;
//import cologne.eck.peafactory.tools.Help;
import cologne.eck.peafactory.tools.ReadResources;


public final class PswDialogImage extends PswDialogBase {

	private static int imageWidth = 800;
	private static int imageHeight = 600;
	protected static boolean resize = true;

	private static PswDialogView dialogView;

	private static PswDialogImage pswDialog;	
	private static LockFrameImage lockFrame = null;	
	
	private static final String VERSION = "0.1";
	private static final String YEAR_OF_PUBLICATION = "2016"; 
	private static final String WEBSITE = "/html/image_pea.html";
	private static final String SOURCE_LINK = "http://eck.cologne/peafactory/src/downloads/imagePEA/imagePEA-0.1-src.tar.gz"; 

	
		
	private PswDialogImage() {		
		
		pswDialog = this;		
		dialogView = PswDialogView.getInstance();
	}
	
	private final static PswDialogImage getInstance() {

		PswDialogImage dialog = null;
		if (pswDialog == null) {

			dialog = new PswDialogImage();
			
		} else {
			// return null
		}
		return dialog;
	}
	
	public static void main(String[] args) {		
		
		if(args.length > 0) {
			PswDialogBase.setWorkingMode(args[0]);
		}		

		initializeVariables();
		
		// settings:
		PswDialogBase.setFileType("image");
		
		Attachments.setFileIdentifier(PeaSettings.getFileIdentifier());

		PswDialogImage pswDialog = PswDialogImage.getInstance();		
		setDialog(pswDialog);		
		
		if (PswDialogView.isInitializing() == true) {
			
			NewPasswordDialog.setRandomCollector(true);
			NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(
					PswDialogImage.dialogView, PswDialogBase.getBundle().getString("initialization"), null);
			pswDialog.setInitializedPassword( newPswDialog.getDialogInput() );
			NewPasswordDialog.setRandomCollector(false);

			if (newPswDialog.getDialogInput() == null){
				System.exit(0);
			} else {
				PswDialogView.setMessage("select_image");
				PswDialogBase.getDialog().startExecution(false);
			}
		} else {
			PswDialogImage.dialogView.setVisible(true);
		}
	}

	
	//-------------------HELPER-FUNCTIONS--------------

	@Override
	public void clearSecretValues() {
		// is done in LockFrameImage to handle initialization
		//LockFrameImage.clearImageBytes();		
	}

	@Override
	public void startDecryption() {
		
		byte[] cipherBytes = null;
		
		if (PswDialogBase.isInsideJar() == true) {
			// don't use File.separator, because it's loaded by Classloader
			cipherBytes = ReadResources.getResourceFromJAR("resources/text.lock");
			
		} else {
			cipherBytes = ReadResources.readExternFile(PswDialogBase.getEncryptedFileName());// there is only one file name
		}		

		// always get the attached salt and use to update programRandomBytes
		handleAttachedSalt(cipherBytes);
			
		byte[] keyMaterial = null;
		if ( PswDialogView.isInitializing() == true ){
			keyMaterial = null;
		} else {
			keyMaterial = getKeyMaterial();
			//Help.printBytes("PswDialogImage startDecryption keyMaterial",keyMaterial);
		}
		
		if (keyMaterial == null && PswDialogView.isInitializing() == false) { // bug
			System.err.println("keyMaterial null");
			return;
		}
		
		//
		// decrypt:
		//	
		byte[] plainBytes = null;
		

		// encrypt key by sessionKey
		plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, keyMaterial, true );

	
		if (plainBytes == null) {
			dialogView.displayErrorMessages(PswDialogBase.getBundle().getString("password_failed"));
			PswDialogView.clearPassword();
			return;
		}		

		LockFrameImage.setImageBytes(plainBytes);

		// display decrypted image: 
		if (lockFrame == null) {
			lockFrame = LockFrameImage.getInstance();
		}

		lockFrame.setTitle(PswDialogBase.getEncryptedFileName());
		lockFrame.setVisible(true);		

		PswDialogView.getView().setVisible(false);	
		EntropyPool.getInstance().stopCollection();
		
		if (PswDialogBase.isInsideJar() == false) {
			pathFileCheck();
		}				
	}
	
	// not used here:
	@Override
	public void preComputeInThread() {}
	@Override
	public String[] getSelectedFileNames() {return null;}

	protected static final void setImageWidth(int newWidth) {
		imageWidth = newWidth;
	}
	protected static final void setImageHeight(int newHeight) {
		imageHeight = newHeight;
	}
	protected static final void setChangeSize(boolean newResize) {
		resize = newResize;
	}
	protected static final int getImageWidth(){
		return imageWidth;		
	}
	protected static final int getImageHeight(){
		return imageHeight;		
	}

	@Override
	public void initializeNewFiles() {
		byte[] imageBytes = null;
		// open FileChooser for Images			
		JFileChooser chooser = new JFileChooser();		
		PswDialogView.customizeFileChooser(PswDialogBase.getBundle().getString(
				"choose_image_to_encrypt"), PswDialogBase.getBundle().getString("select_image_file"), 
				null, null, chooser);

		FileFilter imageFilter = new FileNameExtensionFilter(
			    "Image files", ImageIO.getReaderFileSuffixes());
		chooser.setFileFilter(imageFilter);
		
	    int returnVal = chooser.showOpenDialog(null);
	    
	    if(returnVal == JFileChooser.APPROVE_OPTION) {	// ok

	    	File selectedFile = chooser.getSelectedFile();
	    	String selectedFileName = selectedFile.getAbsolutePath();	
	    	
	    	if (selectedFile.canRead() == false || selectedFile.canWrite() == false) {
				dialogView.displayErrorMessages(PswDialogBase.getBundle().getString("read_write_failed"));
				System.err.println("No access to file: " + selectedFileName);
				System.err.println("Missing access rights, you need more rights for this file");
				//e.printStackTrace();
				return;
	    	}
	    	
	    	// read file (it is plaintext)
	    	imageBytes = ReadResources.readExternFile(selectedFileName);
	    	LockFrameImage.setImageBytes(imageBytes);

	    	KeyDerivation.setSalt(new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));

			byte[] keyMaterial = getKeyMaterial();
			
	    	// set file name in PswDialogBase
	    	PswDialogBase.setEncryptedFileName(selectedFileName);
	    	PswDialogBase.setInsideJar(false);

			LockFrame.saveContent(keyMaterial, imageBytes, selectedFileName);

	    } else { // no file chosen
	    	System.out.println("No file selected");
	    	return;
	    }			
	}

	@Override
	public void openEncryptedFiles() {
		JFileChooser chooser = new JFileChooser();
		
	    PswDialogView.customizeFileChooser(
	    		PswDialogBase.getBundle().getString("open_file"),
	    		PswDialogBase.getBundle().getString("open_encrypted_file"), 
	    		null,
	    		null,
	    		chooser);

		FileFilter imageFilter = new FileNameExtensionFilter(
			    "Image files", ImageIO.getReaderFileSuffixes());

		chooser.setFileFilter( imageFilter );

	    int returnVal = chooser.showOpenDialog(PswDialogView.getView());

	    if(returnVal == JFileChooser.APPROVE_OPTION) {		    
	    	
	    	File file = chooser.getSelectedFile();
	    	String selectedFileName = file.getAbsolutePath();
   	
	    	String checkResult = PswDialogBase.checkFile(selectedFileName, false);
	    	if ( checkResult != null ) {
	    		
	    		System.err.println("selected file failed: " + selectedFileName);
	    		PswDialogView.setMessage(checkResult);
	    		return;

	    	} else { // no error message, file valid
	    		
	    		PswDialogView.addAvailableFile(selectedFileName, true);
	    	}
	    } 		
	}
	@Override
	public String getVersion() {
		return VERSION;
	}
	@Override
	public String getYearOfPublication() {
		return YEAR_OF_PUBLICATION;
	}
	@Override
	public String getWebsite() {
		return WEBSITE;
	}
	@Override
	public String getSourceLink() {
		return SOURCE_LINK;
	}
}	